from __future__ import unicode_literals, absolute_import
from django.db import models
from django.utils.translation import gettext_lazy
from django.conf import settings
from django.apps import apps
import os
import shutil
import zipfile
from .utilities import get_languages, slugify, list_to_string, list_to_dict
from .config import LIST_OF_COUNTRIES, LANGUAGE_DATA
from typing import Union
from uuid import uuid4
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.http import HttpResponse
from django.core.management import call_command
import polib
import xlwt
import xlsxwriter
import pandas
from numpy import nan



class Language(models.Model):

    LIST_OF_COUNTRIES = LIST_OF_COUNTRIES
    LANGUAGE_DATA = LANGUAGE_DATA

    code = models.CharField(primary_key=True, unique=True, max_length=12, verbose_name=gettext_lazy('Language code'))
    name = models.CharField(unique=False, max_length=32, verbose_name=gettext_lazy('Language name'))

    active = models.BooleanField(default=False, verbose_name=gettext_lazy('Active'))
    live = models.BooleanField(default=False, verbose_name=gettext_lazy('Live'))

    country = models.CharField(max_length=20, choices=LIST_OF_COUNTRIES, verbose_name=gettext_lazy('Country'), null=True, blank=True)

    class Meta:
        verbose_name = gettext_lazy('Language')
        verbose_name_plural = gettext_lazy('Languages')

    class InvalidLanguageCode(Exception):

        def __init__(self, code):
            self.message = f"Invalid language: {code}. Please define this language inside the LANGUAGES tuple in your settings file and try again."
            super().__init__(self.message)  

    def __str__(self) -> str:
        return f'{self.name} ({self.code})'
    
    def save(self, *args, **kwargs) -> None:

        languages = get_languages()

        if self.code in languages:
            #if languages[self.code] == self.name:
            if self.active is False and self.live is True:
                self.live = False

            return super().save(*args, **kwargs) 
            
        raise self.InvalidLanguageCode(self)
    
    def delete(self, *args, **kwargs) -> None:
        if self.code not in get_languages():
            return super().delete(*args, **kwargs)
        else:
            self.deactivate()

    def deactivate(self) -> None:
        self.active = False
        self.save()

    @classmethod
    def get_active_languages(cls, name_local=True) -> dict[str, str]:
        '''Returns a dictionary of active languages.'''
        languages = get_languages(name_local=name_local)
        return {language.code: languages[language.code] for language in cls.objects.filter(active=True)}
        #return {language.code: language.name for language in cls.objects.filter(active=True)}
    
    @classmethod
    def get_live_languages(cls, name_local=True) -> dict[str, str]:
        '''Returns a dictionary of live languages.'''
        languages = get_languages(name_local=name_local)
        return {language.code: languages[language.code] for language in cls.objects.filter(live=True)}
        #return {language.code: language.name for language in cls.objects.filter(live=True)}

    @classmethod
    def create_bulk(cls, languages: Union[list,tuple]) -> None:
        '''Creates multiple languages based of an list/tuple input.'''

        if isinstance(languages, (list, tuple)) is False:
            raise TypeError('The input should be a list or a tuple.')

        all_languages = get_languages()
        default_language = apps.get_app_config('django_internationalization').LANGUAGE_CODE

        for value in languages:
            if value in all_languages and not cls.objects.filter(code=value).exists():
                cls.objects.create(
                    code=value, 
                    name=cls.LANGUAGE_DATA[value]['name_local'] if cls.LANGUAGE_DATA[value] else all_languages[value],
                    active=True if value == default_language else False,
                    live=True if value == default_language else False,
                    country=cls.LANGUAGE_DATA[value]['country'] if cls.LANGUAGE_DATA[value]['country'] else None
                )

    @classmethod
    def activate_all_languages(cls, set_live:bool = True) -> None:
        '''Activates all languages.'''

        for language in cls.objects.all():
            language.active = True
            if set_live:
                language.live = True
            language.save()

    @classmethod
    def deactivate_all_languages(cls) -> None:
        '''Deactivates all languages.'''

        for language in cls.objects.all():
            language.deactivate()

class TranslationTag(models.Model):
    
    name = models.CharField(primary_key=True, unique=True, max_length=256, verbose_name=gettext_lazy('Tag'))

    class Meta:
        verbose_name = gettext_lazy('Translation tag')
        verbose_name_plural = gettext_lazy('Translation tags')

    def __str__(self) -> str:
        return self.name
    
    def save(self, *args, **kwargs) -> None:
        self.name = slugify(self.name)
        super().save(*args, **kwargs)

    def delete(self, *args, **kwargs) -> None:
        system_tags = apps.get_app_config('django_internationalization').SYSTEM_TRANSLATION_TAGS

        if not self.name in system_tags:
            super().delete(*args, **kwargs)

    @classmethod
    def create_bulk(cls, tags: Union[list,tuple]) -> None:
        '''Creates multiple tags based of an list/tuple input.'''

        if isinstance(tags, (list, tuple)) is False:
            raise TypeError('The input should be a list or a tuple.')
        
        for tag in tags:
            if not cls.objects.filter(name=tag).exists():
                cls.objects.create(name=tag)
 
class Translation(models.Model):

    key = models.CharField(max_length=128, editable=False, default=None, null=True, blank=True, verbose_name=gettext_lazy('Key'))

    language = models.ForeignKey(Language, on_delete=models.CASCADE, related_name='translations', null=True, blank=True, verbose_name=gettext_lazy('Language'))
    tags = models.ManyToManyField(TranslationTag, related_name='translations', blank=True, verbose_name=gettext_lazy('Tags'))

    content = models.TextField(null=True, blank=True, verbose_name=gettext_lazy('Content'))
    protected = models.BooleanField(default=False, editable=True if getattr(settings, 'DEBUG', True) else False, verbose_name=gettext_lazy('Protected'))

    # Constants
    valid_extensions = ['xls', 'xlsx']

    class Meta:
        verbose_name = gettext_lazy('Translation')
        verbose_name_plural = gettext_lazy('Translations')

    class InvalidLanguageError(Exception):

        def __init__(self, code):
            self.message = f"Invalid language code: {code}. Please create a Language object with that specific code before you proceed."
            super().__init__(self.message)

    class InvalidExtensionError(Exception):

        def __init__(self, extension):
            self.message = f"Invalid file extension: {extension}. The extension should be either .xls or .xlsx."
            super().__init__(self.message)

    class InactiveLanguageError(Exception):
            
            def __init__(self, code):
                languages = get_languages()

                self.message = f"The language with code {languages[code]} ({code}) is not active. Please activate it before you proceed."
                super().__init__(self.message)

    class InvalidUploadError(Exception):

        def __init__(self, message):
            self.message = message
            super().__init__(self.message)

    def __init__(self, *args, **kwargs) -> None:

        if 'valid_extensions' in kwargs:
            del kwargs['valid_extensions']

        super().__init__(*args, **kwargs)

    def __str__(self) -> str:
        return f'{self.key} ({self.language})'
    
    def save(self, sync_copies:bool = True, *args, **kwargs) -> None:

        # Generate a key if it does not exist
        if self.key == None:
            while True:
                key = uuid4().hex
                if not Translation.objects.filter(key=key).exists():
                    self.key = key
                    break

        super().save(*args, **kwargs)
        
        # Create or synchronize copies of the translation instance
        if sync_copies:
            self.__create_and_sync_copies__()

    def delete(self, *args, **kwargs) -> None:
        if not self.protected:
            super().delete(*args, **kwargs)

    @classmethod
    def create(cls, code:str, content:str, tags:Union[list,tuple] = [], key:str = None, protected:bool = False) -> 'Translation':
        '''Creates a new translation object.'''

        languages = Language.get_active_languages()
        if code not in languages:
            raise cls.InvalidLanguageError(code)
        
        for tag in tags:
            TranslationTag.objects.get_or_create(name=tag)

        key = key if key else uuid4().hex
        tags_objects = TranslationTag.objects.filter(name__in=tags)

        translation = cls.objects.create(
            key=key,
            language=Language.objects.get(code=code),
            content=content,
            protected=protected
        )
        translation.tags.set(tags_objects)

    def __create_and_sync_copies__(self, sync_content:bool = False) -> None:
        '''Creates or synchronizes copies of the translation instance.'''
        
        languages = Language.get_active_languages()
        for code in languages:
            language = Language.objects.get(code=code)
            tags = self.tags.all()

            if not Translation.objects.filter(key=self.key, language__code=code).exists():
                translation = Translation.objects.create(
                    key=self.key,
                    language=language,
                    content=self.content,
                    protected=self.protected
                )
                translation.tags.set(self.tags.all())
            else:
                translation = Translation.objects.get(key=self.key, language__code=code)
                translation.protected = self.protected
                translation.content = self.content if sync_content else translation.content
                translation.tags.set(tags)
                translation.save(sync_copies=False)

    @classmethod
    def generate_translation_file(cls, from_language:str, to_languages:Union[str, list[str]], extension:str, get_system_translations:bool = False) -> 'HttpResponse':

        languages = get_languages()
        valid_extensions = cls.valid_extensions

        if type(to_languages) is str:
            to_languages = [to_languages]

        if from_language is None:
            from_language = apps.get_app_config('django_internationalization').LANGUAGE_CODE

        # Check if the from and to languages are valid for the project
        if from_language not in languages:
            raise cls.InvalidLanguageError(from_language)
        if not Language.objects.filter(code=from_language).exists():
            raise cls.InactiveLanguageError(from_language)
        
        for l in to_languages:
            if l not in languages or not Language.objects.filter(code=l).exists():
                raise cls.InvalidLanguageError(l)
            else:
                if Language.objects.get(code=l).active is False:
                    raise cls.InactiveLanguageError(l)

        # Check if the extension is valid
        if extension not in valid_extensions:
            raise cls.InvalidExtensionError()
        
        # Generate the translation file based on extension type
        # (Formats could be expanded in the future)
        if extension in ['xls', 'xlsx']:
            return cls.create_workbook(from_language, to_languages, extension, get_system_translations)
        
    @classmethod
    def create_workbook(cls, from_language:str, to_languages:list[str], extension:str, get_system_translations:bool = False) -> 'HttpResponse':
        '''Creates workbooks for specified language.'''

        files_to_zip = []
        dict_of_languages = get_languages()
        system_tag_objects = TranslationTag.objects.filter(name__in=apps.get_app_config('django_internationalization').SYSTEM_TRANSLATION_TAGS)


        # Create a temporary directory for the files
        translation_path = apps.get_app_config('django_internationalization').TRANSLATION_PATH
        if not os.path.exists(translation_path):
            os.mkdir(translation_path)
        if not os.path.exists(f"{translation_path}/downloads"):
            os.mkdir(f"{translation_path}/downloads")
        temp_path = f"{translation_path}/downloads/{uuid4().hex}"
        os.mkdir(temp_path)

        # Get the translation objects
        if get_system_translations:
            translation_objects = Translation.objects.filter(language__code=from_language)
        else:
            translation_objects = Translation.objects.filter(language__code=from_language).exclude(tags__in=system_tag_objects)

        # Create the workbook for the from language
        for l in to_languages:

            translations = []

            # Translation objects
            for translation in translation_objects:
                translated_content = Translation.objects.get(key=translation.key, language__code=l).content
                translations.append((translation.key, translation.content, translated_content))

            # PO files
            if get_system_translations:
                po_file = polib.pofile(f"{apps.get_app_config('django_internationalization').LOCALE_PATHS[0]}/{l.replace('-', '_')}/LC_MESSAGES/django.po")
                for entry in po_file:
                    translations.append((entry.msgid, entry.msgid, entry.msgstr))

            # Create workbook path and Remove prior workbook if it exists
            file_path = f"{temp_path}/translation_{l}.{extension}"
            if os.path.isfile(file_path):
                os.remove(file_path)

            if extension == 'xls':
                # Sheet styles
                header_style = xlwt.easyxf('font: bold on; align: horiz center; align: vert center; font: bold on, height 280;')
                text_style_locked = xlwt.easyxf('align: vert center; align: wrap on;')
                text_style_unlocked = xlwt.easyxf('align: vert center; align: wrap on;')

                # Create workbook and sheet
                workbook = xlwt.Workbook(encoding='utf-8')
                sheet = workbook.add_sheet('Translations', cell_overwrite_ok=True)

                sheet.write(0, 0, 'ID', header_style)
                sheet.write(0, 1, dict_of_languages[from_language], header_style)
                sheet.write(0, 2, dict_of_languages[l], header_style)

                row = sheet.row(0)
                row.height = 1200
                row.height_mismatch = True

                sheet.col(0).hidden = 1
                sheet.col(1).width = 256 * 60
                sheet.col(2).width = 256 * 60

                # Write translations
                for i, item in enumerate(translations, 1):
                    sheet.write(i, 0, item[0], text_style_locked)
                    sheet.write(i, 1, item[1], text_style_locked)
                    sheet.write(i, 2, item[2], text_style_unlocked)

                workbook.save(file_path)
            elif extension == 'xlsx':
                # Create workbook and sheet
                workbook = xlsxwriter.Workbook(file_path)
                sheet = workbook.add_worksheet('Translations')

                # Styles
                header_format = workbook.add_format({'align': 'center', 'valign': 'vcenter', 'font_size': 20, 'text_wrap': True})
                hidden_header_format = workbook.add_format({'hidden': 1})
                text_format_locked = workbook.add_format({'font_size': 12, 'valign': 'vcenter', 'text_wrap': True})
                text_format_unlocked = workbook.add_format({'font_size': 12, 'valign': 'vcenter', 'text_wrap': True, 'locked': False})

                # Set first row
                sheet.write(0, 0, 'ID', hidden_header_format)
                sheet.write(0, 1, dict_of_languages[from_language], header_format)
                sheet.write(0, 2, dict_of_languages[l], header_format)
                sheet.set_column(0, 0, 20)
                sheet.set_column(1, 2, 40)

                # Write translations
                for i, item in enumerate(translations, 1):
                    sheet.write(i, 0, item[0], text_format_locked)
                    sheet.write(i, 1, item[1], text_format_locked)
                    sheet.write(i, 2, item[2], text_format_unlocked)

                sheet.set_column(0, 0, None, None, {'hidden': True})

                sheet.protect()
                workbook.close()

            files_to_zip.append(file_path)

        # Zip the files
        try:
            zip_file_path = f"{temp_path}/translations.zip"
            with zipfile.ZipFile(zip_file_path, 'w') as zip_file:
                for file in files_to_zip:
                    file_name = os.path.basename(file)
                    arcname = file_name
                    zip_file.write(os.path.join(temp_path, file_name), arcname=arcname)

            with open(zip_file_path, 'rb') as zip_file:
                response = HttpResponse(zip_file.read(), content_type='application/zip')
                response['Content-Disposition'] = f'attachment; filename=translations.zip'
                return response
        finally:
            shutil.rmtree(temp_path)

    @classmethod
    def upload_translation_file(cls, file_path:str) -> None:
        '''Uploads a translation file and updates the database and po/mo files.'''

        languages = get_languages()
        valid_extensions = cls.valid_extensions

        try:
            file_name = os.path.basename(file_path)
            extension = file_name.split('.')[-1]
            lang = file_name.split('.')[0].split('_')[-1]
        except:
            raise cls.InvalidUploadError('Something went wrong while processing the file. Please try again.')
 
        if extension not in valid_extensions:
            raise cls.InvalidExtensionError(f'The file must be one of the following extensions: {list_to_string(valid_extensions)}')
        
        if file_name.split('.')[0].split('_')[0] != 'translation':
            raise cls.InvalidUploadError('Please select the valid translations file.')

        if lang not in languages:
            raise cls.InvalidUploadError('Please select a valid translation file with a valid language code.')
        
        try:
            workbook = pandas.read_excel(file_path, index_col=None, usecols='A,C')
            workbook = workbook.replace(nan, '', regex=True)

            other_translations = []

            for item in workbook.values:
                key = item[0]
                content = item[1]

                if cls.objects.filter(key=key, language__code=lang).exists():
                    translation = cls.objects.get(key=key, language__code=lang)
                    translation.content = content
                    translation.save()
                else:
                    other_translations.extend([key, content])

            other_translations = list_to_dict(other_translations)

            po_file = polib.pofile(f"{apps.get_app_config('django_internationalization').LOCALE_PATHS[0]}/{lang.replace('-', '_')}/LC_MESSAGES/django.po")

            for entry in po_file:
                if entry.msgid in other_translations:
                    entry.msgstr = other_translations[entry.msgid]
            
            po_file.save()

            call_command('compilemessages')
        except:
            raise cls.InvalidUploadError('Something went wrong while processing the file. Please try again.')
        finally:
            if os.path.isfile(file_path):
                os.remove(file_path)
                
    

@receiver(post_save, sender=Language)
def language_activated(sender, instance, created, **kwargs):
    '''Automatically creates translation objects when a language is activated
    and calls the makemessages and compilemessages commands.'''

    if created:
        code = instance.code.replace("-", "_")
        call_command('makemessages', f'--locale={code}')
        call_command('compilemessages', f'--locale={code}')

    if instance.active:
        distinct_keys = Translation.objects.values('key').distinct()
        translations = Translation.objects.filter(key__in=[item['key'] for item in distinct_keys])
        for translation in translations:
            translation.__create_and_sync_copies__(sync_content=True)



class TranslationFile(models.Model):

    file = models.FileField(upload_to=f"{apps.get_app_config('django_internationalization').name}/translations/uploads", verbose_name=gettext_lazy('Translation file'))
    uploaded_at = models.DateTimeField(auto_now_add=True, verbose_name=gettext_lazy('Uploaded at'))

    class Meta:
        verbose_name = gettext_lazy('Translation file')
        verbose_name_plural = gettext_lazy('Translation files')

    def __str__(self) -> str:
        return self.file.name
    


    


from django.core.management.base import BaseCommand
from numpy import std
from ...models import Language
from django.core.management.base import CommandParser
from sys import stdout


class Command(BaseCommand):
    help = '''This command activates and sets all languages to live.'''

    def add_arguments(self, parser: CommandParser) -> None:
        parser.add_argument(
            '--set-live',
            '-sl',
            required=False,
            action='store_true',
            default=False,
            help='Specifies whether to set the languages to live. Default is False.'
        )

    def handle(self, *args, **options) -> None:
        stdout.write('Activating all languages...\n')
        Language.activate_all_languages(set_live=options.get('set_live', False))





from sys import stdout
from django.core.management import call_command
from django.core.management.base import BaseCommand
from ...utilities import get_languages
from ...models import Language, TranslationTag
from django.apps import apps
import os





class Command(BaseCommand):

    help = '''This command prepares the translation module by creating the 
            necessary Language and TranslationTag instances.'''

    def handle(self, *args, **kwargs) -> None:

        call_command('makemigrations')
        call_command('migrate')

        language_codes = [key for key in get_languages().keys()]
        #raise Exception(language_codes)
        Language.create_bulk(language_codes)

        translation_tags = apps.get_app_config('django_internationalization').SYSTEM_TRANSLATION_TAGS
        TranslationTag.create_bulk(translation_tags)

        LOCALE_PATHS = apps.get_app_config('django_internationalization').LOCALE_PATHS

        for path in LOCALE_PATHS:
            if not os.path.exists(path):
                os.makedirs(path)

        self.compile(language_codes)

    def compile(self, language_codes) -> None:
        language_codes = [key.replace('-', '_') for key in get_languages().keys()]
        #try:
        for code in language_codes:
            call_command('makemessages', f'--locale={code}')

        call_command('compilemessages')
        
        #except:
        #    stdout.write('''You need to install GNU text tools v0.15 or newer for this command to work properly.
        #                    Please use the following link to install the necessary tools: https://www.gnu.org/software/gettext/''')




from django.utils.translation import gettext_lazy
import os


# This is a tuple that supposedly will not change
LIST_OF_COUNTRIES : tuple = (
    ('AF', 'Afghanistan'),
    ('AX', 'Åland Islands'),
    ('AL', 'Albania'),
    ('DZ', 'Algeria'),
    ('AS', 'American Samoa'),
    ('AD', 'Andorra'),
    ('AO', 'Angola'),
    ('AI', 'Anguilla'),
    ('AQ', 'Antarctica'),
    ('AG', 'Antigua and Barbuda'),
    ('AR', 'Argentina'),
    ('AM', 'Armenia'),
    ('AW', 'Aruba'),
    ('AU', 'Australia'),
    ('AT', 'Austria'),
    ('AZ', 'Azerbaijan'),
    ('BS', 'Bahamas'),
    ('BH', 'Bahrain'),
    ('BD', 'Bangladesh'),
    ('BB', 'Barbados'),
    ('BY', 'Belarus'),
    ('BE', 'Belgium'),
    ('BZ', 'Belize'),
    ('BJ', 'Benin'),
    ('BM', 'Bermuda'),
    ('BT', 'Bhutan'),
    ('BO', 'Bolivia'),
    ('BQ', 'Bonaire, Sint Eustatius and Saba'),
    ('BA', 'Bosnia and Herzegovina'),
    ('BW', 'Botswana'),
    ('BV', 'Bouvet Island'),
    ('BR', 'Brazil'),
    ('IO', 'British Indian Ocean Territory'),
    ('BN', 'Brunei Darussalam'),
    ('BG', 'Bulgaria'),
    ('BF', 'Burkina Faso'),
    ('BI', 'Burundi'),
    ('CV', 'Cabo Verde'),
    ('KH', 'Cambodia'),
    ('CM', 'Cameroon'),
    ('CA', 'Canada'),
    ('KY', 'Cayman Islands'),
    ('CF', 'Central African Republic'),
    ('TD', 'Chad'),
    ('CL', 'Chile'),
    ('CN', 'China'),
    ('CX', 'Christmas Island'),
    ('CC', 'Cocos (Keeling) Islands'),
    ('CO', 'Colombia'),
    ('KM', 'Comoros'),
    ('CG', 'Congo'),
    ('CD', 'Democratic Republic of the Congo'),
    ('CK', 'Cook Islands'),
    ('CR', 'Costa Rica'),
    ('CI', "Côte d'Ivoire"),
    ('HR', 'Croatia'),
    ('CU', 'Cuba'),
    ('CW', 'Curaçao'),
    ('CY', 'Cyprus'),
    ('CZ', 'Czechia'),
    ('DK', 'Denmark'),
    ('DJ', 'Djibouti'),
    ('DM', 'Dominica'),
    ('DO', 'Dominican Republic'),
    ('EC', 'Ecuador'),
    ('EG', 'Egypt'),
    ('SV', 'El Salvador'),
    ('GQ', 'Equatorial Guinea'),
    ('ER', 'Eritrea'),
    ('EE', 'Estonia'),
    ('SZ', 'Eswatini'),
    ('ET', 'Ethiopia'),
    ('FK', 'Falkland Islands (Malvinas)'),
    ('FO', 'Faroe Islands'),
    ('FJ', 'Fiji'),
    ('FI', 'Finland'),
    ('FR', 'France'),
    ('GF', 'French Guiana'),
    ('PF', 'French Polynesia'),
    ('TF', 'French Southern Territories'),
    ('GA', 'Gabon'),
    ('GM', 'Gambia'),
    ('GE', 'Georgia'),
    ('DE', 'Germany'),
    ('GH', 'Ghana'),
    ('GI', 'Gibraltar'),
    ('GR', 'Greece'),
    ('GL', 'Greenland'),
    ('GD', 'Grenada'),
    ('GP', 'Guadeloupe'),
    ('GU', 'Guam'),
    ('GT', 'Guatemala'),
    ('GG', 'Guernsey'),
    ('GN', 'Guinea'),
    ('GW', 'Guinea-Bissau'),
    ('GY', 'Guyana'),
    ('HT', 'Haiti'),
    ('HM', 'Heard Island and McDonald Islands'),
    ('VA', 'Holy See'),
    ('HN', 'Honduras'),
    ('HK', 'Hong Kong'),
    ('HU', 'Hungary'),
    ('IS', 'Iceland'),
    ('IN', 'India'),
    ('ID', 'Indonesia'),
    ('IR', 'Iran'),
    ('IQ', 'Iraq'),
    ('IE', 'Ireland'),
    ('IM', 'Isle of Man'),
    ('IT', 'Italy'),
    ('JM', 'Jamaica'),
    ('JP', 'Japan'),
    ('JE', 'Jersey'),
    ('JO', 'Jordan'),
    ('KZ', 'Kazakhstan'),
    ('KE', 'Kenya'),
    ('KI', 'Kiribati'),
    ('KP', 'North Korea'),
    ('KR', 'South Korea'),
    ('KW', 'Kuwait'),
    ('KG', 'Kyrgyzstan'),
    ('LA', "Lao People's Democratic Republic"),
    ('LV', 'Latvia'),
    ('LB', 'Lebanon'),
    ('LS', 'Lesotho'),
    ('LR', 'Liberia'),
    ('LY', 'Libya'),
    ('LI', 'Liechtenstein'),
    ('LT', 'Lithuania'),
    ('LU', 'Luxembourg'),
    ('MO', 'Macao'),
    ('MG', 'Madagascar'),
    ('MW', 'Malawi'),
    ('MY', 'Malaysia'),
    ('MV', 'Maldives'),
    ('ML', 'Mali'),
    ('MT', 'Malta'),
    ('MH', 'Marshall Islands'),
    ('MQ', 'Martinique'),
    ('MR', 'Mauritania'),
    ('MU', 'Mauritius'),
    ('YT', 'Mayotte'),
    ('MX', 'Mexico'),
    ('FM', 'Micronesia'),
    ('MD', 'Moldova'),
    ('MC', 'Monaco'),
    ('MN', 'Mongolia'),
    ('ME', 'Montenegro'),
    ('MS', 'Montserrat'),
    ('MA', 'Morocco'),
    ('MZ', 'Mozambique'),
    ('MM', 'Myanmar'),
    ('NA', 'Namibia'),
    ('NR', 'Nauru'),
    ('NP', 'Nepal'),
    ('NL', 'Netherlands'),
    ('NC', 'New Caledonia'),
    ('NZ', 'New Zealand'),
    ('NI', 'Nicaragua'),
    ('NE', 'Niger'),
    ('NG', 'Nigeria'),
    ('NU', 'Niue'),
    ('NF', 'Norfolk Island'),
    ('MK', 'North Macedonia'),
    ('MP', 'Northern Mariana Islands'),
    ('NO', 'Norway'),
    ('OM', 'Oman'),
    ('PK', 'Pakistan'),
    ('PW', 'Palau'),
    ('PS', 'Palestine, State of'),
    ('PA', 'Panama'),
    ('PG', 'Papua New Guinea'),
    ('PY', 'Paraguay'),
    ('PE', 'Peru'),
    ('PH', 'Philippines'),
    ('PN', 'Pitcairn'),
    ('PL', 'Poland'),
    ('PT', 'Portugal'),
    ('PR', 'Puerto Rico'),
    ('QA', 'Qatar'),
    ('RE', 'Réunion'),
    ('RO', 'Romania'),
    ('RU', 'Russia'),
    ('RW', 'Rwanda'),
    ('BL', 'Saint Barthélemy'),
    ('SH', 'Saint Helena, Ascension and Tristan da Cunha'),
    ('KN', 'Saint Kitts and Nevis'),
    ('LC', 'Saint Lucia'),
    ('MF', 'Saint Martin'),
    ('PM', 'Saint Pierre and Miquelon'),
    ('VC', 'Saint Vincent and the Grenadines'),
    ('WS', 'Samoa'),
    ('SM', 'San Marino'),
    ('ST', 'Sao Tome and Principe'),
    ('SA', 'Saudi Arabia'),
    ('SN', 'Senegal'),
    ('RS', 'Serbia'),
    ('SC', 'Seychelles'),
    ('SL', 'Sierra Leone'),
    ('SG', 'Singapore'),
    ('SX', 'Sint Maarten'),
    ('SK', 'Slovakia'),
    ('SI', 'Slovenia'),
    ('SB', 'Solomon Islands'),
    ('SO', 'Somalia'),
    ('ZA', 'South Africa'),
    ('GS', 'South Georgia and the South Sandwich Islands'),
    ('SS', 'South Sudan'),
    ('ES', 'Spain'),
    ('LK', 'Sri Lanka'),
    ('SD', 'Sudan'),
    ('SR', 'Suriname'),
    ('SJ', 'Svalbard and Jan Mayen'),
    ('SE', 'Sweden'),
    ('CH', 'Switzerland'),
    ('SY', 'Syria'),
    ('TW', 'Taiwan'),
    ('TJ', 'Tajikistan'),
    ('TZ', 'Tanzania'),
    ('TH', 'Thailand'),
    ('TL', 'Timor-Leste'),
    ('TG', 'Togo'),
    ('TK', 'Tokelau'),
    ('TO', 'Tonga'),
    ('TT', 'Trinidad and Tobago'),
    ('TN', 'Tunisia'),
    ('TR', 'Turkey'),
    ('TM', 'Turkmenistan'),
    ('TC', 'Turks and Caicos Islands'),
    ('TV', 'Tuvalu'),
    ('UG', 'Uganda'),
    ('UA', 'Ukraine'),
    ('AE', 'United Arab Emirates'),
    ('GB', 'United Kingdom'),
    ('UM', 'United States Minor Outlying Islands'),
    ('US', 'United States of America'),
    ('UY', 'Uruguay'),
    ('UZ', 'Uzbekistan'),
    ('VU', 'Vanuatu'),
    ('VE', 'Venezuela'),
    ('VN', 'Viet Nam'),
    ('VG', 'Virgin Islands (British)'),
    ('VI', 'Virgin Islands (U.S.)'),
    ('WF', 'Wallis and Futuna'),
    ('EH', 'Western Sahara'),
    ('YE', 'Yemen'),
    ('ZM', 'Zambia'),
    ('ZW', 'Zimbabwe'),
    # --------------------------------------
    ('DE-SN', 'Saxony'),
    ('ES-CT', 'Catalonia'),
    ('ES-GA', 'Galicia'),
    ('ES-PV', 'Basque Country'),
    ('GB-WLS', 'Wales'),
    ('ES-VC', 'Valencia'),
)

# Contains data for languages including the name in one language, the name in the local language, and the country code
LANGUAGE_DATA : dict = {
    "af": {"name": gettext_lazy("Afrikaans"), "name_local": "Afrikaans", "country": "ZA"},
    "ar": {"name": gettext_lazy("Arabic"), "name_local": "اَلْعَرَبِيَّةُ", "country": "DZ"},
    "ar-DZ": {"name": gettext_lazy("Algerian Arabic"), "name_local": "الدارجة الجزائرية", "country": "DZ"},
    "ast": {"name": gettext_lazy("Asturian"), "name_local": "asturianu", "country": "ES"},
    "az": {"name": gettext_lazy("Azerbaijani"), "name_local": "Azərbaycan", "country": "AZ"},
    "be": {"name": gettext_lazy("Belarusian"), "name_local": "беларуская", "country": "BY"},
    "bg": {"name": gettext_lazy("Bulgarian"), "name_local": "Български", "country": "BG"},
    "bn": {"name": gettext_lazy("Bengali"), "name_local": "বাংলা", "country": "BD"},
    "br": {"name": gettext_lazy("Breton"), "name_local": "brezhoneg", "country": "FR"},
    "bs": {"name": gettext_lazy("Bosnian"), "name_local": "bosanski", "country": "BA"},
    "ca": {"name": gettext_lazy("Catalan"), "name_local": "català", "country": "AD"},
    "ckb": {"name": gettext_lazy("Central Kurdish (Sorani)"), "name_local": "سۆرانی", "country": "IQ"},
    "cs": {"name": gettext_lazy("Czech"), "name_local": "čeština", "country": "CZ"},
    "cy": {"name": gettext_lazy("Welsh"), "name_local": "Cymraeg", "country": "GB-WLS"},
    "da": {"name": gettext_lazy("Danish"), "name_local": "dansk", "country": "DK"},
    "de": {"name": gettext_lazy("German"), "name_local": "Deutsch", "country": "DE"},
    "dsb": {"name": gettext_lazy("Lower Sorbian"), "name_local": "dolnoserbšćina", "country": "DE"},
    "el": {"name": gettext_lazy("Greek"), "name_local": "Ελληνικά", "country": "GR"},
    "en-US": {"name": gettext_lazy("English"), "name_local": "English", "country": "US"},
    "en-AU": {"name": gettext_lazy("Australian English"), "name_local": "English", "country": "AU"},
    "en-GB": {"name": gettext_lazy("British English"), "name_local": "English", "country": "GB"},
    "eo": {"name": gettext_lazy("Esperanto"), "name_local": "Lingvo Internacia", "country": None},
    "es": {"name": gettext_lazy("Spanish"), "name_local": "español", "country": "ES"},
    "es-AR": {"name": gettext_lazy("Argentinian Spanish"), "name_local": "español", "country": "AR"},
    "es-CO": {"name": gettext_lazy("Colombian Spanish"), "name_local": "español", "country": "CO"},
    "es-MX": {"name": gettext_lazy("Mexican Spanish"), "name_local": "español", "country": "MX"},
    "es-NI": {"name": gettext_lazy("Nicaraguan Spanish"), "name_local": "español", "country": "NI"},
    "es-VE": {"name": gettext_lazy("Venezuelan Spanish"), "name_local": "español", "country": "VE"},
    "et": {"name": gettext_lazy("Estonian"), "name_local": "eesti keel", "country": "EE"},
    "eu": {"name": gettext_lazy("Basque"), "name_local": "euskara", "country": "ES"},
    "fa": {"name": gettext_lazy("Persian"), "name_local": "فارسی", "country": "IR"},
    "fi": {"name": gettext_lazy("Finnish"), "name_local": "suomi", "country": "FI"},
    "fr": {"name": gettext_lazy("French"), "name_local": "français", "country": "FR"},
    "fy": {"name": gettext_lazy("Frisian"), "name_local": "Frysk", "country": "NL"},
    "ga": {"name": gettext_lazy("Irish"), "name_local": "Gaeilge", "country": "IE"},
    "gd": {"name": gettext_lazy("Scottish Gaelic"), "name_local": "Gàidhlig", "country": "GB"},
    "gl": {"name": gettext_lazy("Galician"), "name_local": "galego", "country": "ES"},
    "he": {"name": gettext_lazy("Hebrew"), "name_local": "עִבְֿרִית", "country": "IL"},
    "hi": {"name": gettext_lazy("Hindi"), "name_local": "हिन्दी", "country": "IN"},
    "hr": {"name": gettext_lazy("Croatian"), "name_local": "hrvatski", "country": "HR"},
    "hsb": {"name": gettext_lazy("Upper Sorbian"), "name_local": "hornjoserbšćina", "country": "DE-SN"},
    "hu": {"name": gettext_lazy("Hungarian"), "name_local": "magyar", "country": "HU"},
    "hy": {"name": gettext_lazy("Armenian"), "name_local": "հայերեն", "country": "AM"},
    "ia": {"name": gettext_lazy("Interlingua"), "name_local": "interlingua", "country": None},
    "id": {"name": gettext_lazy("Indonesian"), "name_local": "Bahasa Indonesia", "country": "ID"},
    "ig": {"name": gettext_lazy("Igbo"), "name_local": "Ìgbò", "country": "NG"},
    "io": {"name": gettext_lazy("Ido"), "name_local": "Ido", "country": "FI"},
    "is": {"name": gettext_lazy("Icelandic"), "name_local": "íslenska", "country": "IS"},
    "it": {"name": gettext_lazy("Italian"), "name_local": "italiano", "country": "IT"},
    "ja": {"name": gettext_lazy("Japanese"), "name_local": "日本語", "country": "JP"},
    "ka": {"name": gettext_lazy("Georgian"), "name_local": "ქართული", "country": "GE"},
    "kab": {"name": gettext_lazy("Kabyle"), "name_local": "ⵜⴰⵇⴱⴰⵢⵍⵉⵜ", "country": "DZ"},
    "kk": {"name": gettext_lazy("Kazakh"), "name_local": "қазақ", "country": "KZ"},
    "km": {"name": gettext_lazy("Khmer"), "name_local": "ខ្មែរ", "country": "KH"},
    "kn": {"name": gettext_lazy("Kannada"), "name_local": "ಕನ್ನಡ", "country": "IN"},
    "ko": {"name": gettext_lazy("Korean"), "name_local": "조선말", "country": "KP"}, 
    "ky": {"name": gettext_lazy("Kyrgyz"), "name_local": "Кыргыз", "country": "KG"},
    "lb": {"name": gettext_lazy("Luxembourgish"), "name_local": "Lëtzebuergesch", "country": "LU"},
    "lt": {"name": gettext_lazy("Lithuanian"), "name_local": "lietuvių", "country": "LT"},
    "lv": {"name": gettext_lazy("Latvian"), "name_local": "latviešu", "country": "LV"},
    "mk": {"name": gettext_lazy("Macedonian"), "name_local": "македонски", "country": "MK"},
    "ml": {"name": gettext_lazy("Malayalam"), "name_local": "മലയാളം", "country": "IN"},
    "mn": {"name": gettext_lazy("Mongolian"), "name_local": "монгол", "country": "MN"},
    "mr": {"name": gettext_lazy("Marathi"), "name_local": "मराठी", "country": "IN"},
    "ms": {"name": gettext_lazy("Malay"), "name_local": "Melayu", "country": "MY"},
    "my": {"name": gettext_lazy("Burmese"), "name_local": "မြန်မာ", "country": "MM"},
    "nb": {"name": gettext_lazy("Norwegian Bokmål"), "name_local": "bokmål", "country": "NO"},
    "ne": {"name": gettext_lazy("Nepali"), "name_local": "नेपाली", "country": "NP"},
    "nl": {"name": gettext_lazy("Dutch"), "name_local": "Nederlands", "country": "NL"},
    "nn": {"name": gettext_lazy("Norwegian Nynorsk"), "name_local": "nynorsk", "country": "NO"},
    "os": {"name": gettext_lazy("Ossetic"), "name_local": "ирон ӕвзаг", "country": "RU"},
    "pa": {"name": gettext_lazy("Punjabi"), "name_local": "ਪੰਜਾਬੀ", "country": "PK"},
    "pl": {"name": gettext_lazy("Polish"), "name_local": "polski", "country": "PL"},
    "pt": {"name": gettext_lazy("Portuguese"), "name_local": "português", "country": "PT"},
    "pt-BR": {"name": gettext_lazy("Brazilian Portuguese"), "name_local": "português", "country": "BR"},
    "ro": {"name": gettext_lazy("Romanian"), "name_local": "Română", "country": "RO"},
    "ru": {"name": gettext_lazy("Russian"), "name_local": "Русский", "country": "RU"},
    "sk": {"name": gettext_lazy("Slovak"), "name_local": "slovenský", "country": "SK"},
    "sl": {"name": gettext_lazy("Slovenian"), "name_local": "slovenščina", "country": "SI"},
    "sq": {"name": gettext_lazy("Albanian"), "name_local": "shqip", "country": "AL"},
    "sr": {"name": gettext_lazy("Serbian"), "name_local": "српски", "country": "RS"},
    "sr-LATN": {"name": gettext_lazy("Serbian Latin"), "name_local": "srpski", "country": "RS"},
    "sv": {"name": gettext_lazy("Swedish"), "name_local": "Svenska", "country": "SE"},
    "sw": {"name": gettext_lazy("Swahili"), "name_local": "Kiswahili", "country": "TZ"},
    "ta": {"name": gettext_lazy("Tamil"), "name_local": "தமிழ்", "country": "IN"},
    "te": {"name": gettext_lazy("Telugu"), "name_local": "తెలుగు", "country": "IN"},
    "tg": {"name": gettext_lazy("Tajik"), "name_local": "Тоҷикӣ", "country": "TJ"},
    "th": {"name": gettext_lazy("Thai"), "name_local": "แบบไทย", "country": "TH"},
    "tk": {"name": gettext_lazy("Turkmen"), "name_local": "türkmençe", "country": "TM"},
    "tr": {"name": gettext_lazy("Turkish"), "name_local": "Türkçe", "country": "TR"},
    "tt": {"name": gettext_lazy("Tatar"), "name_local": "татар", "country": "RU"},
    "udm": {"name": gettext_lazy("Udmurt"), "name_local": "Удмурт", "country": "RU"},
    "ug": {"name": gettext_lazy("Uyghur"), "name_local": "ئۇيغۇر", "country": "CN"},
    "uk": {"name": gettext_lazy("Ukrainian"), "name_local": "українська", "country": "UA"},
    "ur": {"name": gettext_lazy("Urdu"), "name_local": "اردو", "country": "PK"},
    "uz": {"name": gettext_lazy("Uzbek"), "name_local": "Oʻzbekcha", "country": "UZ"},
    "vi": {"name": gettext_lazy("Vietnamese"), "name_local": "Tiếng Việt", "country": "VN"},
    "zh-HANS": {"name": gettext_lazy("Simplified Chinese"), "name_local": "汉字", "country": "CN"},
    "zh-HANT": {"name": gettext_lazy("Traditional Chinese"), "name_local": "漢字", "country": "CN"},
    
}

# Tuple of tuples containing language locale and name
LANGUAGES : tuple = tuple((key, value['name']) for key, value in LANGUAGE_DATA.items())
LANGUAGES_LOCAL : tuple = tuple((key, value['name_local']) for key, value in LANGUAGE_DATA.items())

LANGUAGE_CODE : str = 'en-US'

APP_TRANSLATION_PATH : str = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'translations')

APP_LOCALE_PATH : str = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'locale')

SYSTEM_TRANSLATION_TAGS : list = ['system', 'django-admin', 'admin-panel']

# To be added
LANGUAGES_TO_BE_ADDED = ( 
    ("fa_AF", "فارسی"), # Persian (Afghanistan)
    ("ps_AF", "پښتو"), # Pashto (Afghanistan)
    ("sq", "shqip"), # Albanian (Albania)
    ("ar_DZ", "العربيّة"), # Arabic (Algeria)
    ("ber_DZ", "تَمَزِيغت"), # Berber (Algeria)
    ("ca", "català"), # Catalan (Andorra)
    ("pt_AO", "português"), # Portuguese (Angola)
    ("en_AG", "English"), # English (Antigua and Barbuda)
    ("es_AR", "Español"), # Spanish (Argentina)
    ("hy", "հայերէն"), # Armenian (Armenia)
    ("en_AU", "English"), # English (Australia)
    ("de_AT", "Deutsch"), # German (Austria)
    ("az", "Azərbaycanca"), # Azerbaijani (Azerbaijan)
    ("en_BS", "English"), # English (Bahamas)
    ("ar_BH", "العربيّة"), # Arabic (Bahrain)
    ("bn_BD", "বাংলা"), # Bengali (Bangladesh)
    ("en_BB", "English"), # English (Barbados)
    ("be", "Беларуская"), # Belarusian (Belarus)
    ("ru_BY", "русский"), # Russian (Belarus)
    ("nl_BE", "Nederlands"), # Dutch (Belgium)
    ("fr_BE", "Français"), # French (Belgium)
    ("de_BE", "Deutsch"), # German (Belgium)
    ("en_BZ", "English"), # English (Belize)
    ("fr_BJ", "Français"), # French (Benin)
    ("dz", "རྫོང་ཁ་"), # Dzongkha (Bhutan)
    ("es_BO", "castellano"), # Castilian (Bolivia)
    ("aro", "Araona"), # Araona (Bolivia)
    ("cav", "Cavineña"), # Cavineña (Bolivia)
    ("cyb", "Kayuvava"), # Cayubaba (Bolivia)
    ("cao", "Chokobo_Pakawara"), # Chácobo (Bolivia)
    ("qu", "Kechua"), # Quechua (Bolivia)
    ("gn_BO", "avañe'ẽ"), # Guaraní (Bolivia)
    ("bs", "bosanski"), # Bosnian (Bosnia and Herzegovina)
    ("hr_BA", "hrvatski"), # Croatian (Bosnia and Herzegovina)
    ("sr_BA", "српски"), # Serbian (Bosnia and Herzegovina)
    ("en_BW", "English"), # English (Botswana)
    ("pt_BR", "português"), # Portuguese (Brazil)
    ("ms_BN", "بهاس ملايو"), # Malay (Brunei)
    ("bg", "български"), # Bulgarian (Bulgaria)
    ("fr_BF", "Français"), # French (Burkina Faso)
    ("fr_BI", "Français"), # French (Burundi)
    ("rn", "Ikirundi"), # Kirundi (Burundi)
    ("en_BI", "English"), # English (Burundi)
    ("km", "ភាសាខ្មែរ"), # Khmer (Cambodia)
    ("en_CM", "English"), # English (Cameroon)
    ("fr_CM", "Français"), # French (Cameroon)
    ("en_CA", "English"), # English (Canada)
    ("fr_CA", "Français"), # French (Canada)
    ("pt_CV", "português"), # Portuguese (Cape Verde)
    ("fr_CF", "Français"), # French (Central African Republic)
    ("sg", "sängö"), # Sango (Central African Republic)
    ("ar_TD", "العربيّة"), # Arabic (Chad)
    ("fr_TD", "Français"), # French (Chad)
    ("es_CL", "Español"), # Spanish (Chile)
    ("zh", "汉语"), # Chinese (China)
    ("en_CX", "English"), # English (Christmas Island)
    ("cmn_CX", "官话"), # Mandarin (Christmas Island)
    ("ms_CX", "بهاس ملايو"), # Malay (Christmas Island)
    ("en_CC", "English"), # English (Cocos Islands)
    ("coa", "Basa Pulu Cocos"), # Cocos Malay (Cocos Islands)
    ("es_CO", "Español"), # Spanish (Colombia)
    ("ar_KM", "العربيّة"), # Arabic (Comoros)
    ("zdj", "شِكُمُرِ"), # Comorian (Comoros)
    ("wni", "شِكُمُرِ"), # Comorian (Comoros)
    ("swb", "شِكُمُرِ"), # Comorian (Comoros)
    ("wlc", "شِكُمُرِ"), # Comorian (Comoros)
    ("fr_KM", "Français"), # French (Comoros)
    ("fr_CD", "Français"), # French (Democratic Republic of the Congo)
    ("fr_CG", "Français"), # French (Republic of the Congo)
    ("en_CK", "English"), # English (Cook Islands)
    ("mi_CK", "Māori"), # Maori (Cook Islands)
    ("es_CR", "Español"), # Spanish (Costa Rica)
    ("hr", "hrvatski"), # Croatian (Croatia)
    ("es_CU", "Español"), # Spanish (Cuba)
    ("el_CY", "Ελληνικά"), # Greek (Cyprus)
    ("tr_CY", "Türkçe"), # Turkish (Cyprus)
    ("cs", "česky"), # Czech (Czech Republic)
    ("sk_CZ", "slovenčina"), # Slovak (Czech Republic)
    ("da", "dansk"), # Danish (Denmark)
    ("ar_DJ", "العربيّة"), # Arabic (Djibouti)
    ("fr_DJ", "Français"), # French (Djibouti)
    ("en_DM", "English"), # English (Dominica)
    ("es_DO", "Español"), # Spanish (Dominican Republic)
    ("pt_TL", "português"), # Portuguese (East Timor)
    ("tet_TL", "Tetun"), # Tetum (East Timor)
    ("es_EC", "Español"), # Spanish (Ecuador)
    ("ar_EG", "العربيّة"), # Arabic (Egypt)
    ("es_SV", "Español"), # Spanish (El Salvador)
    ("fr_GQ", "Français"), # French (Equatorial Guinea)
    ("pt_GQ", "português"), # Portuguese (Equatorial Guinea)
    ("es_GQ", "Español"), # Spanish (Equatorial Guinea)
    ("ti_ER", "ትግርኛ"), # Tigrinya (Eritrea)
    ("et", "eesti"), # Estonian (Estonia)
    ("en_SZ", "English"), # English (Eswatini)
    ("ss_SZ", "siSwati"), # Swazi (Eswatini)
    ("aa_ET", "Qafar af"), # Afar (Ethiopia)
    ("am", "አማርኛ"), # Amharic (Ethiopia)
    ("om_ET", "Oromoo"), # Oromo (Ethiopia)
    ("so_ET", "اَف سٝومالِ"), # Somali (Ethiopia)
    ("ti_ET", "ትግርኛ"), # Tigrinya (Ethiopia)
    ("en_FJ", "English"), # English (Fiji)
    ("fj", "Vosa Vaka_Viti"), # Fijian (Fiji)
    ("hif", "फ़िजी हिंदी"), # Fiji Hindi (Fiji)
    ("fi", "suomi"), # Finnish (Finland)
    ("sv_FI", "Svenska"), # Swedish (Finland)
    ("fr", "Français"), # French (France)
    ("fr_GA", "Français"), # French (Gabon)
    ("en_GM", "English"), # English (Gambia)
    ("ka", "ქართული ენა"), # Georgian (Georgia)
    ("de", "Deutsch"), # German (Germany)
    ("en_GH", "English"), # English (Ghana)
    ("el", "Ελληνικά"), # Greek (Greece)
    ("en_GD", "English"), # English (Grenada)
    ("es_GT", "Español"), # Spanish (Guatemala)
    ("fr_GN", "Français"), # French (Guinea)
    ("pt_GW", "português"), # Portuguese (Guinea_Bissau)
    ("en_GY", "English"), # English (Guyana)
    ("fr_HT", "Français"), # French (Haiti)
    #("#", "/"), # Creole (Haiti)
    ("es_HN", "Español"), # Spanish (Honduras)
    ("hu", "magyar nyelv"), # Hungarian (Hungary)
    ("is", "Íslenska"), # Icelandic (Iceland)
    ("hi", "हिन्दी"), # Hindi (India)
    ("en_IN", "English"), # English (India)
    ("id", "Indonesia"), # Indonesian (Indonesia)
    ("fa_IR", "فارسی"), # Persian (Iran)
    ("ar_IQ", "العربيّة"), # Arabic (Iraq)
    ("ckb", "کوردی"), # Kurdish (Iraq)
    ("ga", "Gaeilge"), # Irish (Ireland)
    ("en_IE", "English"), # English (Ireland)
    ("he_IL", "עִבְֿרִית"), # Hebrew (Israel)
    ("it", "Italiano"), # Italian (Italy)
    ("fr_CI", "Français"), # French (Ivory Coast)
    ("en_JM", "English"), # English (Jamaica)
    ("ja", "日本語"), # Japanese (Japan)
    ("ar_JO", "العربيّة"), # Arabic (Jordan)
    ("kk", "قازاقشا"), # Kazakh (Kazakhstan)
    ("ru_KZ", "русский"), # Russian (Kazakhstan)
    ("en_KE", "English"), # English (Kenya)
    ("sw_KE", "كِسوَحِيلِ"), # Swahili (Kenya)
    ("en_KI", "English"), # English (Kiribati)
    ("gil", "Kiribati"), # Gilbertese (Kiribati)
    ("ko_KP", "조선말"), # Korean (North Korea)
    ("ko_KR", "한국어"), # Korean (South Korea)
    #("sq_", "shqip"), # Albanian (Kosovo)
    #("#", "српски"), # Serbian (Kosovo)
    ("ar_KW", "العربيّة"), # Arabic (Kuwait)
    ("ky", "قىرعىز تىلى"), # Kyrgyz (Kyrgyzstan)
    ("ru_KG", "русский"), # Russian (Kyrgyzstan)
    ("lo", "ພາສາລາວ"), # Lao (Laos)
    ("lv", "Lettish"), # Latvian (Latvia)
    ("ar_LB", "العربيّة"), # Arabic (Lebanon)
    ("st_LS", "Sesotho"), # Sotho (Lesotho)
    ("en_LS", "English"), # English (Lesotho)
    ("en_LR", "English"), # English (Liberia)
    ("ar_LY", "العربيّة"), # Arabic (Libya)
    ("de_LI", "Deutsch"), # German (Liechtenstein)
    ("lt", "lietuvių"), # Lithuanian (Lithuania)
    ("fr_LU", "Français"), # French (Luxembourg)
    ("de_LU", "Deutsch"), # German (Luxembourg)
    ("lb", "Lëtzebuergesch"), # Luxembourgish (Luxembourg)
    ("fr_MG", "Français"), # French (Madagascar)
    ("mg", "مَلَغَسِ"), # Malagasy (Madagascar)
    ("en_MW", "English"), # English (Malawi)
    ("ny_MW", "Chichewa"), # Chichewa (Malawi)
    ("ms", "بهاس ملايو"), # Malay (Malaysia)
    ("dv", "ދިވެހި"), # Dhivehi (Maldives)
    ("bm", "ߓߡߊߣߊ߲ߞߊ߲"), # Bambara (Mali)
    ("bbo", "Bobo"), # Bobo (Mali)
    ("boz", "Bozo"), # Bozo (Mali)
    ("mey", "حسانية"), # Hassaniya (Mali)
    ("kao", "Xaasongaxango"), # Kassonke (Mali)
    ("mku", "ߡߊ߬ߣߌ߲߬ߞߊ߬ߞߊ߲"), # Maninke (Mali)
    ("myk", "Mamara"), # Minyanka (Mali)
    ("snk", "سࣷونِکَنْخَنّࣹ"), # Soninke (Mali)
    ("taq", "Tafaghist"), # Tamasheq (Mali)
    ("mt", "Malti"), # Maltese (Malta)
    ("en_MT", "English"), # English (Malta)
    ("en_MH", "English"), # English (Marshall Islands)
    ("mh", "Kajin M̧ajel‌̧"), # Marshallese (Marshall Islands)
    ("ar_MR", "العربيّة"), # Arabic (Mauritania)
    ("en_MU", "English"), # English (Mauritius)
    ("mfe", "morisien"), # Morisien (Mauritius)
    ("es_MX", "Español"), # Spanish (Mexico)
    ("en_FM", "English"), # English (Micronesia)
    ("ro_MD", "românește"), # Romanian (Moldova)
    ("fr_MC", "Français"), # French (Monaco)
    ("mn", "ᠮᠣᠩᠭᠣᠯ ᠬᠡᠯᠡ"), # Mongolian (Mongolia)
    ("cnr", "црногорски"), # Montenegrin (Montenegro)
    ("ar_MA", "العربيّة"), # Arabic (Morocco)
    ("ber_MA", "تَمَزِيغت"), # Berber (Morocco)
    ("pt_MZ", "português"), # Portuguese (Mozambique)
    ("my_MM", "မြန်မာဘာသာ"), # Burmese (Myanmar)
    ("en_NA", "English"), # English (Namibia)
    ("en_NR", "English"), # English (Nauru)
    ("na", "Naoero"), # Nauruan (Nauru)
    ("ne", "नेपाली"), # Nepali (Nepal)
    ("nl", "Nederlands"), # Dutch (Netherlands)
    ("en_NZ", "English"), # English (New Zealand)
    ("mi_NZ", "Māori"), # Maori (New Zealand)
    ("es_NI", "Español"), # Spanish (Nicaragua)
    ("fr_NE", "Français"), # French (Niger)
    ("en_NG", "English"), # English (Nigeria)
    ("en_NU", "English"), # English (Niue)
    ("niu", "ko e vagahau Niuē"), # Niuean (Niue)
    ("en_NF", "English"), # English (Norfolk Island)
    ("pih", "Norf'k"), # Norfuk (Norfolk Island)
    ("mk", "македонски"), # Macedonian (North Macedonia)
    ("sq_MK", "shqip"), # Albanian (North Macedonia)
    ("no", "norsk"), # Norwegian (Norway)
    ("ar_OM", "العربيّة"), # Arabic (Oman)
    ("ur_PK", "اُردُو"), # Urdu (Pakistan)
    ("en_PK", "English"), # English (Pakistan)
    ("en_PW", "English"), # English (Palau)
    ("pau", "a tekoi er a Belau"), # Palauan (Palau)
    ("ar_PS", "العربيّة"), # Arabic (Palestine)
    ("es_PA", "Español"), # Spanish (Panama)
    ("en_PG", "English"), # English (Papua New Guinea)
    #("/", "/"), # Hiri Motu (Papua New Guinea)
    ("tpi", "Tok Pisin"), # Tok Pisin (Papua New Guinea)
    ("es_PY", "Español"), # Spanish (Paraguay)
    ("gn_PY", "avañe'ẽ"), # Guaraní (Paraguay)
    ("es_PE", "Español"), # Spanish (Peru)
    ("fil", "Tagalog"), # Filipino (Philippines)
    ("en_PH", "English"), # English (Philippines)
    ("pl", "polski"), # Polish (Poland)
    ("pt", "português"), # Portuguese (Portugal)
    ("ar_QA", "العربيّة"), # Arabic (Qatar)
    ("ro", "românește"), # Romanian (Romania)
    ("ru", "русский"), # Russian (Russia)
    ("en_RW", "English"), # English (Rwanda)
    ("fr_RW", "Français"), # French (Rwanda)
    ("rw", "Ikinyarwanda"), # Kinyarwanda (Rwanda)
    ("sw_RW", "كِسوَحِيلِ"), # Swahili (Rwanda)
    ("ar_EH", "العربيّة"), # Arabic (Sahrawi Arab Democratic Republic)
    ("es_EH", "Español"), # Spanish (Sahrawi Arab Democratic Republic)
    ("en_KN", "English"), # English (Saint Kitts and Nevis)
    ("en_LC", "English"), # English (Saint Lucia)
    ("en_VC", "English"), # English (Saint Vincent and the Grenadines)
    ("en_WS", "English"), # English (Samoa)
    ("sm", "Gagana faʻa Sāmoa"), # Samoan (Samoa)
    ("it_SM", "Italiano"), # Italian (San Marino)
    ("pt_ST", "português"), # Portuguese (São Tomé and Príncipe)
    ("ar_SA", "العربيّة"), # Arabic (Saudi Arabia)
    ("fr_SN", "Français"), # French (Senegal)
    ("sr", "српски"), # Serbian (Serbia)
    ("en_SC", "English"), # English (Seychelles)
    ("fr_SC", "Français"), # French (Seychelles)
    #("/", "/"), # Seychellois Creole (Seychelles)
    ("en_SL", "English"), # English (Sierra Leone)
    ("en_SG", "English"), # English (Singapore)
    ("ms_SG", "بهاس ملايو"), # Malay (Singapore)
    ("cmn_SG", "官话"), # Mandarin Chinese (Singapore)
    ("ta_SG", "தமிழ்"), # Tamil (Singapore)
    ("sk", "slovenčina"), # Slovak (Slovakia)
    ("sl", "slovenščina"), # Slovene (Slovenia)
    ("en_SB", "English"), # English (Solomon Islands)
    ("so", "اَف سٝومالِ"), # Somali (Somalia)
    ("ar_SO", "العربيّة"), # Arabic (Somalia)
    ("af", "Afrikaans"), # Afrikaans (South Africa)
    ("en_ZA", "English"), # English (South Africa)
    ("nr_ZA", "isiNdebele"), # Southern Ndebele (South Africa)
    ("st_ZA", "Sesotho"), # Sotho (South Africa)
    ("ss_ZA", "siSwati"), # Swazi (South Africa)
    ("ts_ZA", "Xitsonga"), # Tsonga (South Africa)
    ("tn_ZA", "Setswana"), # Tswana (South Africa)
    ("ve_ZA", "Tshivenḓa"), # Venda (South Africa)
    ("xh_ZA", "isiXhosa"), # Xhosa (South Africa)
    ("zu", "isiZulu"), # Zulu (South Africa)
    #("#", "/"), # Ossetian (South Ossetia)
    #("ru_", "русский"), # Russian (South Ossetia)
    ("en_SS", "English"), # English (South Sudan)
    ("es_ES", "Español"), # Spanish (Spain)
    ("si", "සිංහල"), # Sinhala (Sri Lanka)
    ("ta_LK", "தமிழ்"), # Tamil (Sri Lanka)
    ("ar_SD", "العربيّة"), # Arabic (Sudan)
    ("en_SS", "English"), # English (Sudan)
    ("nl_SR", "Nederlands"), # Dutch (Suriname)
    ("sv", "Svenska"), # Swedish (Sweden)
    ("fr_CH", "Français"), # French (Switzerland)
    ("de_CH", "Deutsch"), # German (Switzerland)
    ("it_CH", "Italiano"), # Italian (Switzerland)
    ("rm", "romontsch"), # Romansh (Switzerland)
    ("ar_SY", "العربيّة"), # Arabic (Syria)
    ("cmn_TW", "官话"), # Mandarin (Taiwan)
    ("tg", "Тоҷикӣ"), # Tajik (Tajikistan)
    ("sw_TZ", "كِسوَحِيلِ"), # Swahili (Tanzania)
    ("en_TZ", "English"), # English (Tanzania)
    ("th", "ภาษาไทย"), # Thai (Thailand)
    ("fr_TG", "Français"), # French (Togo)
    ("en_TK", "English"), # English (Tokelau)
    ("tkl", "Tokelau"), # Tokelauan (Tokelau)
    ("en_TO", "English"), # English (Tonga)
    ("to", "lea faka_Tonga"), # Tongan (Tonga)
    #("#", "/"), # Moldovan (Transnistria)
    #("ru_", "русский"), # Russian (Transnistria)
    #("#", "/"), # Ukrainian (Transnistria)
    ("en_TT", "English"), # English (Trinidad and Tobago)
    ("ar_TN", "العربيّة"), # Arabic (Tunisia)
    ("tr", "Türkçe"), # Turkish (Turkey)
    ("tk", "تۆرکمنچه"), # Turkmek (Turkmenistan)
    ("tvl", "Tuuvalu"), # Tuvaluan (Tuvalu)
    ("en_TV", "English"), # English (Tuvalu)
    ("en_UG", "English"), # English (Uganda)
    ("sw_UG", "كِسوَحِيلِ"), # Swahili (Uganda)
    ("uk", "українська"), # Ukrainian (Ukraine)
    ("ar_AE", "العربيّة"), # Arabic (United Arab Emirates)
    ("en_GB", "English"), # English (United Kingdom)
    ("en_US", "English"), # English (United States)
    ("es_UY", "Español"), # Spanish (Uruguay)
    ("uz", "اۉزبېکچه"), # Uzbek (Uzbekistan)
    ("en_VU", "English"), # English (Vanuatu)
    ("fr_VU", "Français"), # French (Vanuatu)
    #("#", "/"), # Bislama (Vanuatu)
    ("es_VE", "Español"), # Spanish (Venezuela)
    ("vi", "tiếng Việt"), # Vietnamese (Vietnam)
    ("ar_YE", "العربيّة"), # Arabic (Yemen)
    ("en_ZM", "English"), # English (Zambia)
    ("ny", "Chichewa"), # Chewa (Zimbabwe)
    #("#", "/"), # Chibarwe (Zimbabwe)
    ("en_ZW", "English"), # English (Zimbabwe)
    ("kck", "Ikalanga"), # Kalanga (Zimbabwe)
    ("nmq", "Nanzva"), # Nambya (Zimbabwe)
    ("nd_ZW", "isiNdebele"), # Northern Ndebele (Zimbabwe)
    ("ts_ZW", "Xitsonga"), # Shangani (Zimbabwe)
    ("sn_ZW", "chiShona"), # Shona (Zimbabwe)
    ("st_ZW", "Sesotho"), # Sotho (Zimbabwe)
    ("to_ZW", "lea faka-Tonga"), # Tonga (Zimbabwe)
    ("tn_ZW", "Setswana"), # Tswana (Zimbabwe)
    ("ve_ZW", "Tshivenḓa"), # Venda (Zimbabwe)
    ("xh_ZW", "isiXhosa"), # Xhosa (Zimbabwe)
    ("cy", "Cymraeg"), # Welsh (Britain)
    # ----------------------------------------------
    # Widely spoken languages
    # ----------------------------------------------
    ("it_AL", "Italiano"), # Italian (Albania)
    ("fr_DZ", "Français"), # French (Algeria)
    #("#", "/"), # Creole (Antigua and Barbuda)
    ("ru_AM", "русский"), # Russian (Armenia)
    ("en_AT", "English"), # English (Austria)
    ("ru_AZ", "русский"), # Russian (Azerbaijan)
    ("en_BE", "English"), # English (Belgium)
    ("bzj", "Creole"), # Kriol (Belize)
    ("es_BZ", "Español"), # Spanish (Belize)
    ("sw_BI", "كِسوَحِيلِ"), # Swahili (Burundi)
    ("en_HR", "English"), # English (Croatia)
    ("en_CY", "English"), # English (Cyprus)
    ("en_DK", "English"), # English (Denmark)
    ("en_EG", "English"), # English (Egypt)
    ("ar_ER", "العربيّة"), # Arabic (Eritrea)
    ("it_ER", "Italiano"), # Italian (Eritrea)
    ("ru_EE", "русский"), # Russian (Estonia)
    ("en_FI", "English"), # English (Finland)
    #("#", "/"), # Occitan (France)
    ("ru_GE", "русский"), # Russian (Georgia)
    ("en_DE", "English"), # English (Germany)
    ("ff", "فولا"), # Fula
    #("#", "/"), # Maninka (Guinea)
    ("sus", "Sosoxui"), # Susu (Guinea)
    #("#", "/"), # Guyanese Creole (Guyana)
    ("bn_IN", "বাংলা"), # Bengali (India)
    ("mr", "मराठी"), # Marathi (India)
    ("te", "తెలుగు"), # Telugu (India)
    ("ta_IN", "தமிழ்"), # Tamil (India)
    ("gu_IN", "ગુજરાતી"), # Gujarati (India)
    ("ur_IN", "اُردُو"), # Urdu (India)
    ("kn", "ಕನ್ನಡ"), # Kannada (India)
    ("or", "ଓଡ଼ିଆ"), # Odia (India)
    ("ms", "بهاس ملايو"), # Malayalam (India)
    ("pa_IN", "ਪੰਜਾਬੀ"), # Punjabi (India)
    ("ms", "بهاس ملايو"), # Malay (Indonesia)
    #("#", "/"), # Javanese (Indonesia)
    #("#", "/"), # Sundanese (Indonesia)
    #("#", "/"), # Madurese (Indonesia)
    #("#", "/"), # Minangkabau (Indonesia)
    ("ru_IL", "русский"), # Russian (Israel)
    ("en_IL", "English"), # English (Israel)
    ("ru_LV", "русский"), # Russian (Latvia)
    ("en_LB", "English"), # English (Lebanon)
    ("fr_LB", "Français"), # French (Lebanon)
    ("pl_LT", "polski"), # Polish (Lithuania)
    ("ru_LT", "русский"), # Russian (Lithuania)
    ("en_LU", "English"), # English (Luxembourg)
    ("pt_LU", "português"), # Portuguese (Luxembourg)
    ("cmn_MY", "官话"), # Mandarin Chinese (Malaysia)
    ("yue_MY", "廣東話"), # Cantonese (Malaysia)
    ("ta_MY", "தமிழ்"), # Tamil (Malaysia)
    ("en_MY", "English"), # English (Malaysia)
    ("en_MV", "English"), # English (Maldives)
    ("fr_MR", "Français"), # French (Mauritania)
    #("#", "munegascu"), # Monégasque (Monaco)
    ("en_MM", "English"), # English (Myanmar)
    ("en_NL", "English"), # English (Netherlands)
    ("ha", "هَرْشٜىٰن هَوْسَا"), # Hausa (Nigeria)
    ("en_NO", "English"), # English (Norway)
    ("pa_PK", "پنجابی"), # Punjabi (Pakistan)
    ("bal", "بلۏچی"), # Balochi (Pakistan)
    ("sd_PK", "سِنڌِي"), # Sindhi (Pakistan)
    ("sd_IN", "सिन्धी"), # Sindhi (India)
    ("ps_PK", "پښتو"), # Pashto (Pakistan)
    ("skr", "سرائیکی"), # Saraiki (Pakistan)
    ("en_PS", "English"), # English (Palestine)
    ("he_PS", "עִבְֿרִית"), # Hebrew (Palestine)
    ("en_PL", "English"), # English (Poland)
    ("en_PT", "English"), # English (Portugal)
    ("fil_SA", "Tagalog"), # Filipino (Saudi Arabia)
    ("bn_SA", "বাংলা"), # Bengali (Saudi Arabia)
    ("en_LK", "English"), # English (Sri Lanka)
    ("ru_TJ", "русский"), # Russian (Tajikistan)
    ("my_TH", "မြန်မာဘာသာ"), # Burmese (Thailand)
    #("#", "/"), # Creole (Trinidad and Tobago)
    ("ru_TM", "русский"), # Russian (Turkmenistan)
    ("en_AE", "English"), # English (United Arab Emirates)
    ("es_US", "Español"), # Spanish (United States)
    ("ru_UZ", "русский"), # Russian (Uzbekistan)

    # Other Django languages ------------------------------------------------

)







from django.apps import AppConfig
from django.conf import settings
from django.utils.translation import gettext_lazy
from .config import LANGUAGE_CODE, LANGUAGES, LANGUAGES_LOCAL, APP_TRANSLATION_PATH, APP_LOCALE_PATH, SYSTEM_TRANSLATION_TAGS

_LOCALE_PATHS = []
_LOCALE_PATHS.append(APP_LOCALE_PATH)

class DjangoInternationalizationConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'django_internationalization'
    verbose_name = gettext_lazy('Internationalization')

    def ready(self) -> None:

        self.LANGUAGE_CODE = getattr(settings, 'LANGUAGE_CODE', LANGUAGE_CODE)
        self.LANGUAGES = getattr(settings, 'LANGUAGES', LANGUAGES)
        self.LANGUAGES_LOCAL = getattr(settings, 'LANGUAGES_LOCAL', LANGUAGES_LOCAL)

        # This is the path where the excel translation files are stored
        self.TRANSLATION_PATH = getattr(settings, 'TRANSLATION_PATH', APP_TRANSLATION_PATH)

        # This is a list of paths where the .po and .mo translation files are stored
        self.LOCALE_PATHS = getattr(settings, 'LOCALE_PATHS', [])
        if self.LOCALE_PATHS == []:
            self.LOCALE_PATHS = _LOCALE_PATHS

        # This is a list of tags that are used to identify system translations
        self.SYSTEM_TRANSLATION_TAGS = getattr(settings, 'SYSTEM_TRANSLATION_TAGS', SYSTEM_TRANSLATION_TAGS)

        # [Used for testing purposes] This is a list of all attributes of the settings module
        self.all_attributes = dir(settings)

        return super().ready()




from typing import Union
from django.apps import apps
import re
from django.utils.translation import activate 
from .config import LANGUAGE_DATA, LANGUAGES, LANGUAGES_LOCAL


def list_to_dict(list: Union[list,tuple]) -> dict:
    '''
    Take a list or a tuple with an *even number* of objects and converts it 
    to a dictionary.
    '''

    if len(list) % 2 != 0:
        raise IndexError('Argument does not contain an even number of objects.')

    output = {}

    for i in range(0, len(list), 2):
        output[str(list[i])] = list[i+1]

    return output

def flatten_list(list: list) -> list:
    '''Flattens a list of lists.'''
    return [item for sublist in list for item in sublist]

def slugify(input:str) -> str:
    '''Slugifies input and returns an URL friendly string.'''
    
    input = re.sub(r'[^a-zA-Z0-9_-]', '-', str(input))
    input = input.replace(' ', '_').replace('\n', '_').replace('\t', '_').lower()

    return input

def list_to_string(list: Union[list,tuple]) -> str:
    '''Converts a list to a string.'''
    
    return str(list).replace('[', '').replace(']', '').replace('\'', '')

def get_languages(name_local=True) -> dict:
    '''
    Returns a dictionary of languages from the settings file.
    Format of dictionary: {'locale': 'name'}
    '''
    if name_local is True:
        return list_to_dict(flatten_list(apps.get_app_config('django_internationalization').LANGUAGES_LOCAL))
    else:
        return list_to_dict(flatten_list(apps.get_app_config('django_internationalization').LANGUAGES))
        
def set_language(request, language:str|None = None) -> None:
    '''Activates a language for translation.'''

    default_language = apps.get_app_config('django_internationalization').LANGUAGE_CODE
    all_languages = get_languages()

    l = language if language is not None else request.POST.get('language', request.session.get('language', default_language))
    l = l if l in all_languages else default_language

    l.replace('_', '-')

    activate(l)

    if request.session.get('language', None) != l:
        request.session['language'] = l
        request.session.save()


from django.shortcuts import render
from django.http import HttpResponse
from django.contrib import messages
from django_internationalization.forms import LanguageSelector, TranslationFilesDownloadForm, TranslationFilesUploadForm
from django_internationalization.models import Language, Translation, TranslationFile
from django_internationalization.utilities import set_language
from django.http import HttpResponseRedirect
from django.utils.translation import gettext_lazy
from django.views import View

    
class Test(View):

    def dispatch(self, request, *args, **kwargs) -> HttpResponse:

        return super().dispatch(request, *args, **kwargs)

    def get(self, request, *args, **kwargs) -> HttpResponse:
        context = {}

        context['language_selector'] = LanguageSelector()
        context['translation_files_download_form'] = TranslationFilesDownloadForm()
        context['translation_files_upload_form'] = TranslationFilesUploadForm()
        context['test'] = request.session.get('language', 'kurec')

        return render(request, 'base.html', context)
        
    def post(self, request, *args, **kwargs) -> HttpResponse:
        context = {}

        context['language_selector'] = LanguageSelector(request.POST)
        context['translation_files_download_form'] = TranslationFilesDownloadForm()
        context['translation_files_upload_form'] = TranslationFilesUploadForm()
        context['test'] = request.POST

        return render(request, 'base.html', context)

class ChangeLanguage(View):

    def post(self, request, *args, **kwargs) -> HttpResponse:
        set_language(request)
        return HttpResponseRedirect(request.META.get('HTTP_REFERER'))

class DownloadTranslation(View):

    def post(self, request, *args, **kwargs) -> HttpResponse:


        # from_language = request.POST.get('from_language', request.session['language'] if 'language' in request.session else apps.get_app_config('django_internationalization').LANGUAGE_CODE)
        from_language = request.POST.get('from_language', None)
        if request.POST.get('all_languages', None):
            to_languages = Language.get_active_languages()
        else:
            to_languages = request.POST.getlist('to_languages')

        if not from_language or not to_languages:
            messages.error(request, gettext_lazy('Please select a source language and at least one target language to download the translation files for.'))
            return HttpResponseRedirect(request.META.get('HTTP_REFERER'))
        
        return Translation.generate_translation_file(
            from_language = from_language,
            to_languages = to_languages,
            extension = 'xlsx',
            get_system_translations = request.POST.get('get_system_translations', False)
        )

class UploadTranslation(View):

    def post(self, request, *args, **kwargs) -> HttpResponse:

        try:
            if request.FILES.get('translation_file', None):
                file = TranslationFile.objects.create(file=request.FILES['translation_file'])
                Translation.upload_translation_file(file.file.path)
                file.delete()
                messages.success(request, gettext_lazy('The translation file has been uploaded successfully.'))
            else:
                messages.error(request, gettext_lazy('Please select a translation file to upload'))
        except:
            messages.error(request, gettext_lazy('The translation file was not uploaded.'))


        return HttpResponseRedirect(request.META.get('HTTP_REFERER'))

        


from django import forms
from django.utils.translation import gettext
from .models import Language, TranslationFile
from .utilities import get_languages
from django.apps import apps
from django.utils import translation


class LanguageSelector(forms.ModelForm):

    id = 'language-selector'

    # Behavior of the form
    open_on_hover = True
    open_on_click = True

    # Sets the class of the form. The default is 'default'
    style = 'default' 

    class Meta:
        model = Language
        fields = []

    class CustomModelChoiceField(forms.ModelChoiceField):

        def label_from_instance(self, obj) -> str:
            languages = get_languages()
            try:
                return languages[obj.code]
            except:
                return obj.name
            
    class CustomRadioSelect(forms.RadioSelect):

        flag_paths = []
        active_lang_name = None
        active_lang_code = None

        def __init__(self, *args, **kwargs):
            self.flag_paths.clear()
            super().__init__(*args, **kwargs)


        def create_option(self, name, value, label, selected, index, subindex=None, attrs=None):
            option_dict = super().create_option(name, value, label, selected, index, subindex=subindex, attrs=attrs)

            if value is not None:
                instance = self.choices.queryset.get(pk=value)

                # Add the flag path for the static string call in the template
                # You can change this value if you want to use a different path for your flags
                flag = f"{apps.get_app_config('django_internationalization').name}/flags/{instance.country}.svg" if instance.country else f"{apps.get_app_config('django_internationalization').name}/flags/NO-FLAG.svg"
                #if not flag in self.flag_paths:
                self.flag_paths.append(flag)

                # You can put any attribute you want here and they will appear in the input tag
                option_dict['attrs']['id'] = f'language-selector-code-{instance.code}'
                option_dict['attrs']['class'] = f"language-selector-item {'active' if instance.code == translation.get_language() else ''}"

                if instance.code == translation.get_language():
                    self.active_lang_name = instance.name
                    self.active_lang_code = instance.code


            return option_dict
        
    def __init__(self, *args, **kwargs):
        self.active_languages = Language.get_live_languages()
        super().__init__(*args, **kwargs)

    def _get_language_(self):
        '''Returns the current language code.'''
        
        l = translation.get_language()
        if '-' in l:
            return f"{l.split('-')[0]}-{l.split('-')[1].upper()}"
        else:
            return l

    def display(self) -> bool:
        return Language.get_live_languages().__len__() > 1
    
    def get_active_language(self) -> str:
        return self._get_language_()
    
    def get_active_language_flag(self) -> str:
        try:
            return Language.LANGUAGE_DATA[self._get_language_()]['country']
        except KeyError:
            return 'NO-FLAG'
    
    def get_active_language_name(self) -> str:
        return Language.objects.get(code=self._get_language_()).name

    
    language = CustomModelChoiceField(
        queryset = Language.objects.filter(live=True),
        widget = CustomRadioSelect(
            attrs = {
                'onchange': 'this.form.submit()',
                #'onclick': 'this.form.submit()',
                #'role': 'button',
                'tabindex': '-1'
            },
        ),
    )

class TranslationFilesUploadForm(forms.ModelForm):

    form_text = gettext('''
                    <p class="instruction">Click here or drag file to upload</p>
                    <p class="limitations">Only xls and xlsx files</p>
                   ''')
    
    no_file_text = gettext('- No file selected -')
    
    submit_text = gettext('Upload translation')

    upload_svg_path = f"{apps.get_app_config('django_internationalization').name}/upload.svg"
    close_svg_path = f"{apps.get_app_config('django_internationalization').name}/close-circle.svg"

    class Meta:
        model = TranslationFile
        fields = []

    translation_file = forms.FileField(
        widget = forms.FileInput(
            attrs = {
                'accept': 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet, application/vnd.ms-excel',
                'id': 'translation_file',
                'class': 'translation-upload-input',
                'tabindex': '-1'
            },
        ),
        label = None,
        required = False,
    )

class TranslationFilesDownloadForm(forms.ModelForm):

    submit_text = gettext('Download translation files')
    dropdown_placeholder = gettext('Select language')
    form_header = gettext('Download translation files')

    active_languages = []

    class Meta:
        model = Language
        fields = []

    class CustomModelChoiceField(forms.ModelChoiceField):

        def label_from_instance(self, obj) -> str:
            languages = get_languages(name_local=True)
            try:
                return languages[obj.code]
            except:
                return obj.name

    class CustomRadioSelect(forms.RadioSelect):

        def create_option(self, name, value, label, selected, index, subindex=None, attrs=None):
            option_dict = super().create_option(name, value, label, selected, index, subindex=subindex, attrs=attrs)

            if value is not None:
                instance = self.choices.queryset.get(pk=value)

                # You can put any attribute you want here and they will appear in the input tag
                option_dict['attrs']['id'] = f'language-selector-code-{instance.code}'
                option_dict['attrs']['class'] = f"language-selector-item {'active' if instance.code == translation.get_language() else ''}"




            return option_dict
                
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.active_languages = Language.get_active_languages(name_local=False)

    from_language = forms.ChoiceField(
        choices = active_languages,
        required=True
    )
    
    to_languages = forms.MultipleChoiceField(
        choices = active_languages,
        required=True,
    )

    all_languages = forms.BooleanField(
        required = False,
        widget = forms.CheckboxInput(
            attrs = {
                'class': 'toggle-switch',
                'tabindex': '-1'
            }
        ),
        label = None
    )
    all_languages.content_after = gettext('<p>Download all languages</p>')

    get_system_translations = forms.BooleanField(
        required = False,
        widget = forms.CheckboxInput(
            attrs = {
                'class': 'toggle-switch',
                'tabindex': '-1'
            }
        ),
        label = None
    )
    get_system_translations.content_after = gettext('<p>Include system translations</p>')



from sys import stdout
from django.core.management.base import BaseCommand
from ...models import Language


class Command(BaseCommand):
    help = '''This command deactivates all languages.'''

    def handle(self, *args, **options) -> None:
        stdout.write('Deactivating all languages...\n')
        Language.deactivate_all_languages()





from django.contrib import admin
from .models import Language, TranslationTag, Translation, TranslationFile
from django.conf import settings



class LanguageAdmin(admin.ModelAdmin):

    list_display = ('active', 'live', 'name', 'code', 'country')
    search_fields = ('active', 'live', 'name', 'code', 'country')

    readonly_fields = () if getattr(settings, 'DEBUG', True) else ('name', 'code')

    filter_horizontal = ()
    list_filter = ()

    def get_queryset(self, request):
        queryset = super().get_queryset(request)
        return queryset.order_by('-live', '-active')

    def get_actions(self, request):
        actions = super().get_actions(request)

        if 'delete_selected' in actions:
            del actions['delete_selected']
        
        return actions

admin.site.register(Language, LanguageAdmin)


class TranslationTagAdmin(admin.ModelAdmin):

    list_display = ('name',)
    search_fields = ('name',)

    readonly_fields = ()

    filter_horizontal = ()
    list_filter = ()
    
admin.site.register(TranslationTag, TranslationTagAdmin)


class TranslationAdmin(admin.ModelAdmin):

    list_display = ('key', 'language', 'protected')
    search_fields = ('language__name',)

    readonly_fields = () if getattr(settings, 'DEBUG', True) else ('key', 'language', 'protected')

    filter_horizontal = ()
    list_filter = ()

    def save_model(self, request, obj, form, change) -> None:

        if not change:
            return super().save_model(request, obj, form, change)
        else:
            try:
                return super().save_model(request, obj, form, change)
            finally:
                instance = Translation.objects.get(pk=obj.pk)
                instance.tags.set(form.cleaned_data['tags'])
                instance.save()

admin.site.register(Translation, TranslationAdmin)

'''
class TranslationFileAdmin(admin.ModelAdmin):

    list_display = ('file',)
    search_fields = ('file',)

    readonly_fields = ()

    filter_horizontal = ()
    list_filter = ()

admin.site.register(TranslationFile, TranslationFileAdmin)
'''




from django.urls import path
from .views import Test, ChangeLanguage, UploadTranslation, DownloadTranslation



urlpatterns = [
    #path('', Test.as_view(), name='test'),
    path('change-language', ChangeLanguage.as_view(), name='change_language'),
    path('upload-translation', UploadTranslation.as_view(), name='upload_translation'),
    path('download-translation', DownloadTranslation.as_view(), name='download_translation'),
]


from setuptools import setup, find_packages

with open("./README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='django_internationalization',
    version='1.3',
    packages=find_packages(),
    install_requires=[
        "setuptools>=69.5.1", 
        "Django>=5.0.4", 
        "numpy>=1.26.4", 
        "openpyxl>=3.1.2", 
        "pandas>=2.2.2", 
        "polib>=1.2.0", 
        "XlsxWriter>=3.2.0", 
        "xlwt>=1.3.0"
    ],
    entry_points={
        'console_scripts': [
            'django_internationalization = django_internationalization.__main__:main'
        ]
    },
    long_description=long_description,
    long_description_content_type="text/markdown",
    package_data={'': ['static/*', 'templates/*']}
)

